###############################################################
# W Doug Bolden's Simple "Vigenere" Python System
#	version 0.1
#	April 20, 2007
# 
# ==What's this?==
# 
# The Vignere system is a simple, but moderately effective,
# algorithm for adding a keyword (or keystream) to plain text
# to make what seems to be a random spilling of gibberish but
# is relatively easy to break by those that need the keyword.
#
# It is more secure than a monoalphabetic substitution (i.e.
# "cryptogram"), and more secure than a single shift cipher
# (which is mostly just another word for MAS). In fact, for 
# ease of use, speed of use, and relative ease of transporting
# the key, it is one of the most effective and secure.
#
# Do not overestimate it's security, though. To initiates, the
# output seems unbreakably confusing, but in the end it is
# merely a matter of algebraic solution (as well as other
# methods, depending on some factors). Any time there are more
# equations than factors in an algebraic system, there is a
# definite, if difficult, solution. To answer this, you can 
# increase the keyword/stream size to as large as, or bigger
# than, the plaintext. Of course, doing so infringes on the
# ease of transporting your key. Not to mention, if a key is
# used more than once, it begins to greatly decrease security.
# In this, as in all things, a compromise will be needed.
###############################################################

import random

###############################################################
# ==How to Use this File
# 
# This file is a "python" file, which means you need the 
# excellent Python (this is a programming language) installed. 
# This is not hard. You go to http://www.python.org and down-
# load the new edition. In my opinion, it and Perl handle just
# about everything the average computer user would ever want to
# program/script just fine and they are "cross-platform" for
# the most part. Everything used in this code should be cross-
# platform but hopes are made to be broken and so I might have
# to fix some thing from time to time.
#
# To run this file, you double click it, for Windows, or can run
# from the commandline if you prefer. I would recommend you use
# the IDLE that comes with the install, as it seems to handle 
# output and input better. I use Linux, and run it from the 
# commandline with no problem.
#
# It will ask you four questions. The first question is the
# PLAINTEXT. You can enter a file name or a line of text (of
# most any length). It will first assume what you enter is a 
# filename and try to open it. If this fails, it will assume 
# what you typed was the plaintext itself. If you mistype the
# filename, it will fail the opening and then Vignere the file-
# name as plaintext. In future versions, I may change this.
#
# The second question it will ask you is for the KEY. Again,
# you can type either a filename (it will assume this first) or
# the key itself. The same issue above is true here, as well.
#
# The third question asked with be an OUTPUT "FORMAT". The 
# process will get rid of original spaces (anything by letters)
# and so any of the output formats are technically equal, but
# the options are A (default) which spits out in little groups
# of five letters (this is something of a standard and helps to
# break it up if you want to post it in a medium that might be
# wary of long lines), B which spits it out as one long line
# (you can then insert your own breaks), C (experimental) 
# which will try to break it up in to a reasonable "word space"
# for you, mostly as an obfuscation, and D which will spit it 
# out as one long line but will append "yyy" to the front. The
# reasons for this will be explained in a second.
#
# The final question asked is for an output file name. Note, I
# use a simple "append" command here which means that if a file
# exists it will simply add to the final line of the file. This
# can actually be used to your advantage as certain 
# formats (graphics I know, possibly others) can have text 
# appended to their end with no loss of quality, meaning you
# can transport the final text in several formats and it does
# not have to be an obvious text transfer. The issue here is 
# knowing how to retrieve the text, especially since certain
# viewers add little bits to the end for various reasons. One
# solution is it looks until it finds a line with "yyy" (hence
# option D, above) and then reads the rest of the line. This is
# has some problems, but is a quick work around for right this
# second. You can also just assume the last line, which, will
# work most of the time, but that is for later.
#
# If, for any reason, the file name does not work, it will 
# default to simply outputing it as a standard output.
#
# The rest is the actions of the program and the output, which
# you can e-mail, post to message boards, text message, or 
# even just write out and send to anyone.
#
###############################################################

plaint = raw_input("What do you wish to encipher? (defaults to filename 'plain.txt')  --> ")
plainstring = ""
try:
	if plaint is "":
		plaint = 'plain.txt'
	plainfile = open(plaint)
	while 1:
		plainline = plainfile.readline()
		if not plainline:
			break

		plainlen = len(plainline)
		lineplace = 0
		while lineplace < plainlen:
			if 96 < ord(plainline[lineplace]) < 123:
				plainord = ord(plainline[lineplace])
				plainord = plainord - 32
				plainstring = plainstring + chr(plainord)
			if 64 < ord(plainline[lineplace]) < 91:
				plainstring = plainstring + plainline[lineplace]
			lineplace = lineplace + 1
except:
	lineplace = 0
	while lineplace < len(plaint):
		if 96 < ord(plaint[lineplace]) < 123:
			plainord = ord(plaint[lineplace])
			plainord = plainord - 32
			plainstring = plainstring + chr(plainord)
		if 64 < ord(plaint[lineplace]) < 91:
			plainstring = plainstring + plaint[lineplace]
		lineplace = lineplace + 1

try:
	print "Using " + plainstring[0:10] + "... as plaintext."
except:
	print "Using " + plaintring + "as plaintext."


keys = raw_input("What is the key for this file (defaults to filename 'key.txt'?  --> ")
keystring = ""
try:
	if keys is "":
		keys = 'key.txt'
	keyfile = open(keys)
	while 1:
		keyline = keyfile.readline()
		if not keyline:
			break

		keylen = len(keyline)
		lineplace = 0
		while lineplace < keylen:
			if 96 < ord(keyline[lineplace]) < 123:
				keyord = ord(keyline[lineplace])
				keyord = keyord - 32
				keystring = keystring + chr(keyord)
			if 64 < ord(keyline[lineplace]) < 91:
				keystring = keystring + keyline[lineplace]
			lineplace = lineplace + 1
except:
	lineplace = 0
	while lineplace < len(keys):
		if 96 < ord(keys[lineplace]) < 123:
			keyord = ord(keys[lineplace])
			keyord = keyord - 32
			keystring = keystring + chr(keyord)
		if 64 < ord(keys[lineplace]) < 91:
			keystring = keystring + keys[lineplace]
		lineplace = lineplace + 1

try:
	print "Using " + keystring[0:10] + "... as keystream."
except:
	print "Using " + keystring + "as keystream."


outform = raw_input("What format do you want the output? A(default): fivegroup, B: long line, C: fake words or D: 'yyy'.  --> ")
if outform is "A":
	formout = 1
	print "Using groups of five"
if outform is 'a':
	formout = 1
	print "Using groups of five"
elif outform is "B":
	formout = 2
	print "Using long line"
elif outform is "b":
	formout = 2
	print "Using long line"
elif outform is "C":
	print "Using fake words"
	formout = 3
elif outform is "c":
	print "Using fake words"
	formout = 3
elif outform is "D":
	formout = 4
	print "Prefixing long line with 'yyy'."
elif outform is "d":
	formout = 4
	print "Prefixing long line with 'yyy'."
else:
	formout = 1
	print "Defaulting back to choice 'A' (groups of five)."

outf = raw_input("What file name do you want to output to? (Default is standard output)  --> ")

try:
	outfile = open(outf, 'a')
	outname = 1
except:
	outname = 0

###############################################################

ks = 0
ps = 0
cipherstring = ""
while ps < len(plainstring):
	if ks == len(keystring):
		ks = 0
	
	keyord = ord(keystring[ks]) - 65
	plainord = ord(plainstring[ps]) - 65

	cipherord = plainord + keyord
	while cipherord > 25:
		cipherord = cipherord - 26

	cipherstring = cipherstring + chr(cipherord+65)

	ks = ks + 1
	ps = ps + 1

###############################################################

outstring = ""
if formout is 1:
	place = -5
	while place < len(cipherstring):
		place = place + 5
		try: 
			groupstring = cipherstring[place:place+5]
			if len(outstring) > 0:
				outstring = outstring + " " + groupstring
			else:
				outstring = groupstring
		except:
			groupstring = cipherstring[place:]
			if len(outstring) > 0:
				outstring = outstring + " " + groupstring
			else:
				oustring = groupstring

if formout is 2:
	outstring = cipherstring

wordsize = 0
if formout is 3:
	place = 0
	while place < len(cipherstring):
		wordchance = random.randint(1,100)
		if wordsize == 1:
			wordchance = wordchance + 15
		if wordsize == 2:
			wordchance = wordchance + 10
		if wordsize == 3:
			wordsize = wordsize + 5
		if 1 < wordchance < 5:
			wordsize = 1
		if 6 < wordchance < 15:
			wordsize = 2
		if 16 < wordchance < 30:
			wordsize = 3
		if 31 < wordchance < 45:
			wordsize = 4
		if 46 < wordchance < 65:
			wordsize = 5
		if 61 < wordchance < 75:
			wordsize = 6
		if 76 < wordchance < 90:
			wordsize = 7
		if 91 < wordchance < 96:
			wordsize = 8
		if 97 < wordchance < 100:
			wordsize = 9

		if len(cipherstring[place:]) < 7:
			wordsize = len(cipherstring[place:])

		try:
			wordstring = cipherstring[place:place+wordsize]
			if len(outstring) > 0:
				outstring = outstring + " " + wordstring
			else:
				outstring = wordstring
		except:
			wordstring = cipherstring[place:]
			if len(outstring) > 0:
				outstring = outstring + " " + wordstring
			else: 
				oustring = wordstring

		place = place + wordsize

if formout is 4:
	outstring = "yyy" + cipherstring

###############################################################

if outname is 1:
	print >> outfile, outstring

if outname is 0:
	print outstring
